Attribute VB_Name = "mLogicalDrives"
Option Explicit

Public Type VOLUME_INFORMATION
   RootPath As String 'eg "c:\"
   VolumeLabel As String
   VolumeSerial As String
   FileSystem As String
   Futures As String
   TotalBytes As Currency
   FreeBytes As Currency
   AvailableBytes As Currency
End Type

Private Declare Function GetDiskFreeSpaceEx Lib "kernel32" Alias "GetDiskFreeSpaceExA" (ByVal lpDirectoryName As String, lpFreeBytesAvailableToCaller As Currency, lpTotalNumberOfBytes As Currency, lpTotalNumberOfFreeBytes As Currency) As Long
Private Declare Function GetLogicalDriveStrings Lib "kernel32" Alias "GetLogicalDriveStringsA" (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long
Private Declare Function GetVolumeInformation Lib "kernel32" Alias "GetVolumeInformationA" (ByVal lpRootPathName As String, ByVal lpVolumeNameBuffer As String, ByVal nVolumeNameSize As Long, lpVolumeSerialNumber As Long, lpMaximumComponentLength As Long, lpFileSystemFlags As Long, ByVal lpFileSystemNameBuffer As String, ByVal nFileSystemNameSize As Long) As Long
Private Declare Function ReadFile Lib "kernel32" (ByVal hFile As Long, lpBuffer As Any, ByVal nNumberOfBytesToRead As Long, lpNumberOfBytesRead As Long, ByVal lpOverlapped As Long) As Long
Private Const FS_CASE_SENSITIVE = &H1
Private Const FS_CASE_IS_PRESERVED = &H2
Private Const FS_UNICODE_STORED_ON_DISK = &H4
Private Const FS_PERSISTENT_ACLS = &H8
Private Const FS_FILE_COMPRESSION = &H10
Private Const FS_VOL_IS_COMPRESSED = &H8000

Private Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal ulOptions As Long, ByVal samDesired As Long, phkResult As Long) As Long
Private Declare Function RegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, lpType As Long, lpData As Any, lpcbData As Long) As Long
Private Declare Function RegCloseKey Lib "advapi32.dll" (ByVal hKey As Long) As Long
Private Const REG_BINARY As Long = 3
Private Const KEY_QUERY_VALUE As Long = &H1
Private Const HKEY_LOCAL_MACHINE As Long = &H80000002

Public Function GetDriveLogicalVolumes(ByVal signature As Long, vi() As VOLUME_INFORMATION) As Long
   Dim nCount As Long, ret As Long, i As Long
   Dim aVolumes() As String
   Dim sDrives As String
   Dim regsig As Long
   
   ret = GetLogicalDriveStrings(0, sDrives)
   If ret = 0 Then Exit Function
   sDrives = String(ret, 0)
   ret = GetLogicalDriveStrings(ret, sDrives)
   If ret = 0 Then Exit Function
   sDrives = Left(sDrives, ret - 2)
   aVolumes = Split(sDrives, Chr(0))
   For i = 0 To UBound(aVolumes)
       regsig = GetVolumeSignature(aVolumes(i))
       If regsig = signature Then
          ReDim Preserve vi(nCount)
          vi(nCount) = GetVolumeInfo(aVolumes(i))
          nCount = nCount + 1
       End If
   Next i
   GetDriveLogicalVolumes = nCount
End Function

Private Function GetVolumeSignature(ByVal sRootPath As String) As Long
   Dim hKey As Long, dwSize As Long, lpType As Long
   Dim sValue As String
   Dim abData() As Byte
   
   If sRootPath = "" Or sRootPath = vbNullString Then Exit Function
   Call RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SYSTEM\MountedDevices", 0, KEY_QUERY_VALUE, hKey)
   If hKey = 0 Then Exit Function
   sValue = "\DosDevices\" & Left(sRootPath, 1) & ":"
   Call RegQueryValueEx(hKey, sValue, 0&, lpType, ByVal 0&, dwSize)
   If (lpType <> REG_BINARY) Or (dwSize < 4) Then GoTo RegError
   ReDim abData(dwSize - 1)
   If RegQueryValueEx(hKey, sValue, 0&, lpType, abData(0), dwSize) Then GoTo RegError
   CopyMemory GetVolumeSignature, abData(0), 4
RegError:
   RegCloseKey hKey
End Function

Private Function GetVolumeInfo(ByVal sRootPath As String) As VOLUME_INFORMATION
   Dim vi As VOLUME_INFORMATION
   Dim sVolumeLabel As String, sVolumeSerial As String, sFileSystem As String, s As String
   Dim lVolumeSerial As Long, lTemp As Long, lFlags As Long
   
   If sRootPath = "" Or sRootPath = vbNullString Then Exit Function
   sRootPath = Left(sRootPath, 1) & ":\"
   sVolumeLabel = String(255, 0)
   sFileSystem = String(255, 0)
   GetVolumeInformation sRootPath, sVolumeLabel, Len(sVolumeLabel), lVolumeSerial, lTemp, lFlags, sFileSystem, Len(sFileSystem)
   GetDiskFreeSpaceEx sRootPath, vi.AvailableBytes, vi.TotalBytes, vi.FreeBytes
   vi.RootPath = sRootPath
   vi.FileSystem = TrimNull(sFileSystem)
   vi.VolumeLabel = TrimNull(sVolumeLabel)
   If UCase(vi.FileSystem) = "NTFS" Then
      vi.VolumeSerial = GetNTFSVolumeSerial(sRootPath)
   End If
   If vi.VolumeSerial = "" Then
      s = HexZeroPad(lVolumeSerial, 8)
      s = Left(s, 4) & "-" & Right(s, 4)
      vi.VolumeSerial = s
   End If
   s = ""
   If lFlags And FS_CASE_IS_PRESERVED Then s = s & "Case is preserved" & vbCrLf
   If lFlags And FS_CASE_SENSITIVE Then s = s & "File system is case sensitive" & vbCrLf
   If lFlags And FS_UNICODE_STORED_ON_DISK Then s = s & "Filenames are stored as Unicode" & vbCrLf
   If lFlags And FS_PERSISTENT_ACLS Then s = s & "Access control is supported." & vbCrLf
   If lFlags And FS_FILE_COMPRESSION Then s = s & "Individual files may be compressed" & vbCrLf
   If lFlags And FS_VOL_IS_COMPRESSED Then s = s & "Entire volume is compressed" & vbCrLf
   s = Left(s, Len(s) - 2)
   vi.Futures = s
   GetVolumeInfo = vi
End Function

Private Function GetNTFSVolumeSerial(ByVal sRootPath As String) As String
   Dim hDevice As Long, nRead As Long
   Dim int64_SN As LARGE_INTEGER
   Dim abBuffer(511) As Byte
   hDevice = CreateFile("\\.\" & UCase(Left(sRootPath, 1)) & ":", GENERIC_READ Or GENERIC_WRITE, FILE_SHARE_READ Or FILE_SHARE_WRITE, 0&, OPEN_EXISTING, 0&, 0&)
   If hDevice = INVALID_HANDLE_VALUE Then Exit Function
   Call ReadFile(hDevice, abBuffer(0), 512, nRead, 0&)
   CopyMemory int64_SN, abBuffer(&H48), 8
   CloseHandle hDevice
   GetNTFSVolumeSerial = HexZeroPad(int64_SN.lowpart, 8) & "-" & HexZeroPad(int64_SN.highpart, 8)
End Function
