Attribute VB_Name = "mSMARTDef"
Option Explicit

Public Const MAX_IDE_DRIVES = 4         ' // Max number of drives assuming primary/secondary, master/slave topology
Public Const READ_ATTRIBUTE_BUFFER_SIZE = 512
Public Const IDENTIFY_BUFFER_SIZE = 512
Public Const READ_THRESHOLD_BUFFER_SIZE = 512
Public Const OUTPUT_DATA_SIZE = IDENTIFY_BUFFER_SIZE + 16

Public Declare Function GetLastError Lib "kernel32" () As Long

Public Type SRB_IO_CONTROL
    HeaderLength As Long
    signature(7) As Byte
    Timeout      As Long
    ControlCode  As Long
    ReturnCode   As Long
    Length       As Long
End Type

'IOCTL commands
Public Const DFP_GET_VERSION = &H74080
Public Const DFP_SEND_DRIVE_COMMAND = &H7C084
Public Const DFP_RECEIVE_DRIVE_DATA = &H7C088
'---------------------------------------------------------------------
' GETVERSIONOUTPARAMS contains the data returned from the
' Get Driver Version function.
'---------------------------------------------------------------------
Public Type GETVERSIONOUTPARAMS
       bVersion       As Byte ' Binary driver version.
       bRevision      As Byte ' Binary driver revision.
       bReserved      As Byte ' Not used.
       bIDEDeviceMap  As Byte ' Bit map of IDE devices.
       fCapabilities  As Long ' Bit mask of driver capabilities.
       dwReserved(3)  As Long ' For future use.
End Type

'Bits returned in the fCapabilities member of GETVERSIONOUTPARAMS
Public Const CAP_IDE_ID_FUNCTION = 1             ' ATA ID command supported
Public Const CAP_IDE_ATAPI_ID = 2                ' ATAPI ID command supported
Public Const CAP_IDE_EXECUTE_SMART_FUNCTION = 4  ' SMART commannds supported

'---------------------------------------------------------------------
' IDE registers
'---------------------------------------------------------------------
Public Type IDEREGS
   bFeaturesReg     As Byte ' // Used for specifying SMART "commands".
   bSectorCountReg  As Byte ' // IDE sector count register
   bSectorNumberReg As Byte ' // IDE sector number register
   bCylLowReg       As Byte ' // IDE low order cylinder value
   bCylHighReg      As Byte ' // IDE high order cylinder value
   bDriveHeadReg    As Byte ' // IDE drive/head register
   bCommandReg      As Byte ' // Actual IDE command.
   bReserved        As Byte ' // reserved for future use.  Must be zero.
End Type

'---------------------------------------------------------------------
' SENDCMDINPARAMS contains the input parameters for the
' Send Command to Drive function.
'---------------------------------------------------------------------
Public Type SENDCMDINPARAMS
   cBufferSize     As Long     ' Buffer size in bytes
   irDriveRegs     As IDEREGS  ' Structure with drive register values.
   bDriveNumber    As Byte     ' Physical drive number to send command to (0,1,2,3).
   bReserved(2)    As Byte     ' Bytes reserved
   dwReserved(3)   As Long     ' DWORDS reserved
   bBuffer()      As Byte      ' Input buffer.
End Type

' Valid values for the bCommandReg member of IDEREGS.
Public Const IDE_ATAPI_ID = &HA1               ' Returns ID sector for ATAPI.
Public Const IDE_ID_FUNCTION = &HEC            ' Returns ID sector for ATA.
Public Const IDE_EXECUTE_SMART_FUNCTION = &HB0 ' Performs SMART cmd.
                                               ' Requires valid bFeaturesReg,
                                               ' bCylLowReg, and bCylHighReg

' Cylinder register values required when issuing SMART command
Public Const SMART_CYL_LOW = &H4F
Public Const SMART_CYL_HI = &HC2

'---------------------------------------------------------------------
' Status returned from driver
'---------------------------------------------------------------------
Public Type DRIVERSTATUS
   bDriverError  As Byte          ' Error code from driver, or 0 if no error.
   bIDEStatus    As Byte          ' Contents of IDE Error register.
                                  ' Only valid when bDriverError is SMART_IDE_ERROR.
   bReserved(1)  As Byte
   dwReserved(1) As Long
 End Type

' bDriverError values
Public Enum DRIVER_ERRORS
       SMART_NO_ERROR = 0         ' No error
       SMART_IDE_ERROR = 1        ' Error from IDE controller
       SMART_INVALID_FLAG = 2     ' Invalid command flag
       SMART_INVALID_COMMAND = 3  ' Invalid command byte
       SMART_INVALID_BUFFER = 4   ' Bad buffer (null, invalid addr..)
       SMART_INVALID_DRIVE = 5    ' Drive number not valid
       SMART_INVALID_IOCTL = 6    ' Invalid IOCTL
       SMART_ERROR_NO_MEM = 7     ' Could not lock user's buffer
       SMART_INVALID_REGISTER = 8 ' Some IDE Register not valid
       SMART_NOT_SUPPORTED = 9    ' Invalid cmd flag set
       SMART_NO_IDE_DEVICE = 10   ' Cmd issued to device not present
                                  ' although drive number is valid
       ' 11-255 reserved
End Enum
'---------------------------------------------------------------------
' The following struct defines the interesting part of the IDENTIFY
' buffer:
'---------------------------------------------------------------------
Public Type IDSECTOR
   wGenConfig                   As Integer 'Word 0
   wNumCyls                     As Integer 'Word 1
   wReserved                    As Integer 'Word 2
   wNumHeads                    As Integer 'Word 3
   wBytesPerTrack               As Integer 'Word 4
   wBytesPerSector              As Integer 'Word 5
   wSectorsPerTrack             As Integer 'Word 6
   wVendorUnique(2)             As Integer 'Word 7-9
   sSerialNumber(19)            As Byte    'Word 10-19
   wBufferType                  As Integer 'Word 20
   wBufferSize                  As Integer 'Word 21
   wECCSize                     As Integer 'Word 22
   sFirmwareRev(7)              As Byte    'Word 23-26
   sModelNumber(39)             As Byte    'Word 27-46
   wMoreVendorUnique            As Integer 'Word 47
   wDoubleWordIO                As Integer 'Word 48
   wCapabilities                As Integer 'Word 49
   wReserved1                   As Integer 'Word 50
   wPIOTiming                   As Integer 'Word 51
   wDMATiming                   As Integer 'Word 52
   wBS                          As Integer 'Word 53
   wNumCurrentCyls              As Integer 'Word 54
   wNumCurrentHeads             As Integer 'Word 55
   wNumCurrentSectorsPerTrack   As Integer 'Word 56
   wCurrentSectorCapacity(1)    As Integer 'Word 57-58
   wMultSectorStuff             As Integer 'Word 59
   wTotalAddressableSectors(1)  As Integer 'Word 60-61
   wSingleWordDMA               As Integer 'Word 62
   wMultiWordDMA                As Integer 'Word 63
   wAdvancedPIOMode             As Integer 'Word 64
   wMinDMATime                  As Integer 'Word 65
   wRecommendDMATime            As Integer 'Word 66
   wMinPIOTime                  As Integer 'Word 67
   wMinPIOTimeIORDY             As Integer 'Word 68
   wReserved2(5)                As Integer 'Word 69-74
   wQueueDepth                  As Integer 'Word 75
   wReserved3(3)                As Integer 'Word 76-79
   wMajorVersion                As Integer 'word 80
   wMinorVersion                As Integer 'word 81
   wCommandSetSupported(2)      As Integer 'Word 82-84
   wCommandSetEnabled(2)        As Integer 'Word 85-87
   wUltraDMA                    As Integer 'Word 88
   wSecurityEraseTime           As Integer 'Word 89
   wEnhSecurityEraseTime        As Integer 'Word 90
   wAdvPowerManagement          As Integer 'Word 91
   wMasterPassword              As Integer 'Word 92
   wHardwareReset               As Integer 'Word 93
   wAcousticData                As Integer 'Word 94
   wStreamMinRequestSize        As Integer 'Word 95
   wStreamTransferTimeDMA       As Integer 'Word 96
   wStreamAccessLatency         As Integer 'Word 97
   wStreamPerfGranularity(1)    As Integer 'Word 98-99
   wMaxUserLBA48(3)             As Integer 'Word 100-103
   wStreamTransferTimePIO       As Integer 'Word 104
   wReserved4                   As Integer 'Word 105
   wPhysSectorSize              As Integer 'Word 106
   wInterSeekDelay              As Integer 'Word 107
   wUniqueID(3)                 As Integer 'Word 108-111
   wReserved5(4)                As Integer 'Word 112-116
   wWordsPerLogicalSector(1)    As Integer 'Word 117-118
   wReserved6(7)                As Integer 'Word 119-126
   wRemovableMediaNotify        As Integer 'Word 127
   wSecurityStatus              As Integer 'Word 128
   wVendorSpecific(30)          As Integer 'Word 129-159
   wCFAPowerMode                As Integer 'Word 160
   wReserved7(14)               As Integer 'Word 161-175
   wCurrentMediaSN(29)          As Integer 'Word 176-205
   wReserved8(48)               As Integer 'Word 206-254
   wIntegrityWord               As Integer 'Word 255
End Type

'---------------------------------------------------------------------
' Structure returned by SMART IOCTL for several commands
'---------------------------------------------------------------------
Public Type SENDCMDOUTPARAMS
  cBufferSize   As Long         ' Size of bBuffer in bytes (IDENTIFY_BUFFER_SIZE in our case)
  DRIVERSTATUS  As DRIVERSTATUS ' Driver status structure.
  bBuffer()    As Byte          ' Buffer of arbitrary length in which to store the data read from the drive.
End Type

'---------------------------------------------------------------------
' Feature register defines for SMART "sub commands"
'---------------------------------------------------------------------

Public Const SMART_READ_ATTRIBUTE_VALUES = &HD0
Public Const SMART_READ_ATTRIBUTE_THRESHOLDS = &HD1
Public Const SMART_ENABLE_DISABLE_ATTRIBUTE_AUTOSAVE = &HD2
Public Const SMART_SAVE_ATTRIBUTE_VALUES = &HD3
Public Const SMART_EXECUTE_OFFLINE_IMMEDIATE = &HD4
' Vendor specific commands:
Public Const SMART_ENABLE_SMART_OPERATIONS = &HD8
Public Const SMART_DISABLE_SMART_OPERATIONS = &HD9
Public Const SMART_RETURN_SMART_STATUS = &HDA

'---------------------------------------------------------------------
' The following structure defines the structure of a Drive Attribute
'---------------------------------------------------------------------

Public Const NUM_ATTRIBUTE_STRUCTS = 30

Public Type DRIVEATTRIBUTE
       bAttrID As Byte         ' Identifies which attribute
       wStatusFlags As Integer ' Integer, see bit definitions below
       bAttrValue As Byte      ' Current normalized value
       bWorstValue As Byte     ' How bad has it ever been?
       bRawValue(5) As Byte    ' Un-normalized value
       bReserved As Byte       ' ...
End Type
'---------------------------------------------------------------------
' Status Flags Values
'---------------------------------------------------------------------
Public Enum STATUS_FLAGS
       PRE_FAILURE_WARRANTY = &H1
       ON_LINE_COLLECTION = &H2
       PERFORMANCE_ATTRIBUTE = &H4
       ERROR_RATE_ATTRIBUTE = &H8
       EVENT_COUNT_ATTRIBUTE = &H10
       SELF_PRESERVING_ATTRIBUTE = &H20
End Enum

'---------------------------------------------------------------------
' The following structure defines the structure of a Warranty Threshold
' Obsoleted in ATA4!
'---------------------------------------------------------------------

Public Type ATTRTHRESHOLD
       bAttrID As Byte            ' Identifies which attribute
       bWarrantyThreshold As Byte ' Triggering value
       bReserved(9) As Byte       ' ...
End Type

'---------------------------------------------------------------------
' Known Valid Attribute IDs
'---------------------------------------------------------------------
Public Enum ATTRIBUTE_ID
       ATTR_INVALID = 0
       ATTR_READ_ERROR_RATE = 1
       ATTR_THROUGHPUT_PERF = 2
       ATTR_SPIN_UP_TIME = 3
       ATTR_START_STOP_COUNT = 4
       ATTR_REALLOC_SECTOR_COUNT = 5
       ATTR_READ_CHANNEL_MARGIN = 6
       ATTR_SEEK_ERROR_RATE = 7
       ATTR_SEEK_TIME_PERF = 8
       ATTR_POWER_ON_HRS_COUNT = 9
       ATTR_SPIN_RETRY_COUNT = 10
       ATTR_CALIBRATION_RETRY_COUNT = 11
       ATTR_POWER_CYCLE_COUNT = 12
       ATTR_SOFT_READ_ERROR_RATE = 13
       ATTR_G_SENSE_ERROR_RATE = 191
       ATTR_POWER_OFF_RETRACT_CYCLE = 192
       ATTR_LOAD_UNLOAD_CYCLE_COUNT = 193
       ATTR_TEMPERATURE = 194
       ATTR_HARDWARE_ECC_RECOVERED = 195
       ATTR_REALLOCATION_EVENTS_COUNT = 196
       ATTR_CURRENT_PENDING_SECTOR_COUNT = 197
       ATTR_UNCORRECTABLE_SECTOR_COUNT = 198
       ATTR_ULTRADMA_CRC_ERROR_RATE = 199
       ATTR_WRITE_ERROR_RATE = 200
       ATTR_SOFT_READ_ERROR_RATEI = 201
       ATTR_DATA_ADDRESS_MARKS_ERRORS = 202
       ATTR_RUN_OUT_CANCEL = 203
       ATTR_SOFT_ECC_CORRECTION = 204
       ATTR_THERMAL_ASPERITY_RATE = 205
       ATTR_FLYING_HEIGHT = 206
       ATTR_SPIN_HEIGHT_CURRENT = 207
       ATTR_SPIN_BUZZ = 208
       ATTR_OFFLINE_SEEK_PERFORMANCE = 209
       ATTR_DISK_SHIFT = 220
       ATTR_G_SENSE_ERROR_RATEII = 221
       ATTR_LOADED_HOURS = 222
       ATTR_LOAD_UNLOAD_RETRY_COUNT = 223
       ATTR_LOAD_FRICTION = 224
       ATTR_LOAD_UNLOAD_CYCLE_COUNTII = 225
       ATTR_LOAD_IN_TIME = 226
       ATTR_TORQUE_AMPLIFICATION_COUNT = 227
       ATTR_POWER_OFF_RETRACT_COUNT = 228
       ATTR_GMR_HEAD_AMPLITUDE = 230
       ATTR_TEMPERATUREII = 231
       ATTR_HEAD_FLYING_HOURS = 240
       ATTR_READ_ERROR_RETRY_RATE = 250
End Enum


